#!/usr/bin/env python3
#
# Author: Yipeng Sun <syp at umd dot edu>
# License: BSD 2-clause
# Last Change: Sat Sep 07, 2019 at 12:49 PM -0400

name = 'pyBabyMaker'
version = '0.0.1'  # This version is fixed.
version = "0.2.3.3"
